import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { Disposable, IDisposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { IObservable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/observable";
import { URI } from "@codingame/monaco-vscode-api/vscode/vs/base/common/uri";
import { ICommandService } from "@codingame/monaco-vscode-api/vscode/vs/platform/commands/common/commands.service";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILoggerService } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log.service";
import { INotificationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/notification/common/notification.service";
import { IOpenerService } from "@codingame/monaco-vscode-api/vscode/vs/platform/opener/common/opener.service";
import { StorageScope } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage";
import { IStorageService } from "@codingame/monaco-vscode-api/vscode/vs/platform/storage/common/storage.service";
import { ITelemetryService } from "@codingame/monaco-vscode-api/vscode/vs/platform/telemetry/common/telemetry.service";
import { IWorkspaceContextService } from "@codingame/monaco-vscode-api/vscode/vs/platform/workspace/common/workspace.service";
import { IExtensionService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/extensions/common/extensions.service";
import { IOutputService } from "@codingame/monaco-vscode-api/vscode/vs/workbench/services/output/common/output.service";
import { ToolProgress } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/chat/common/languageModelToolsService";
import { IMcpRegistry } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/mcp/common/mcpRegistryTypes.service";
import { McpServerRequestHandler } from "./mcpServerRequestHandler.js";
import { IMcpServer, IMcpServerConnection, IMcpTool, McpCollectionReference, McpConnectionState, McpDefinitionReference, McpServerDefinition, McpServerToolsState } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/mcp/common/mcpTypes";
import { MCP } from "./modelContextProtocol.js";
interface IToolCacheEntry {
    readonly nonce: string | undefined;
    readonly tools: readonly IValidatedMcpTool[];
}
interface IServerCacheEntry {
    readonly servers: readonly McpServerDefinition.Serialized[];
}
export declare class McpServerMetadataCache extends Disposable {
    private didChange;
    private readonly cache;
    private readonly extensionServers;
    constructor(scope: StorageScope, storageService: IStorageService);
    reset(): void;
    getTools(definitionId: string): IToolCacheEntry | undefined;
    storeTools(definitionId: string, nonce: string | undefined, tools: readonly IValidatedMcpTool[]): void;
    getServers(collectionId: string): IServerCacheEntry | undefined;
    storeServers(collectionId: string, entry: IServerCacheEntry | undefined): void;
}
interface IValidatedMcpTool extends MCP.Tool {
    serverToolName: string;
}
export declare class McpServer extends Disposable implements IMcpServer {
    readonly collection: McpCollectionReference;
    readonly definition: McpDefinitionReference;
    private readonly _requiresExtensionActivation;
    private readonly _toolCache;
    private readonly _mcpRegistry;
    private readonly _extensionService;
    private readonly _loggerService;
    private readonly _outputService;
    private readonly _telemetryService;
    private readonly _commandService;
    private readonly _instantiationService;
    private readonly _notificationService;
    private readonly _openerService;
    private readonly _connectionSequencer;
    private readonly _connection;
    readonly connection: import("@codingame/monaco-vscode-api/vscode/vs/base/common/observable").ISettableObservable<IMcpServerConnection | undefined, void> & IDisposable;
    readonly connectionState: IObservable<McpConnectionState>;
    private get toolsFromCache();
    private readonly toolsFromServerPromise;
    private readonly toolsFromServer;
    readonly tools: IObservable<readonly IMcpTool[]>;
    readonly toolsState: IObservable<McpServerToolsState>;
    private readonly _loggerId;
    private readonly _logger;
    get trusted(): IObservable<boolean | undefined>;
    constructor(collection: McpCollectionReference, definition: McpDefinitionReference, explicitRoots: URI[] | undefined, _requiresExtensionActivation: boolean | undefined, _toolCache: McpServerMetadataCache, _mcpRegistry: IMcpRegistry, workspacesService: IWorkspaceContextService, _extensionService: IExtensionService, _loggerService: ILoggerService, _outputService: IOutputService, _telemetryService: ITelemetryService, _commandService: ICommandService, _instantiationService: IInstantiationService, _notificationService: INotificationService, _openerService: IOpenerService);
    showOutput(): void;
    start(isFromInteraction?: boolean): Promise<McpConnectionState>;
    private showInteractiveError;
    stop(): Promise<void>;
    private resetLiveData;
    private _normalizeTool;
    private _getValidatedTools;
    private populateLiveData;
    callOn<R>(fn: (handler: McpServerRequestHandler) => Promise<R>, token?: CancellationToken): Promise<R>;
}
export declare class McpTool implements IMcpTool {
    private readonly _server;
    private readonly _definition;
    readonly id: string;
    get definition(): MCP.Tool;
    constructor(_server: McpServer, idPrefix: string, _definition: IValidatedMcpTool);
    call(params: Record<string, unknown>, token?: CancellationToken): Promise<MCP.CallToolResult>;
    callWithProgress(params: Record<string, unknown>, progress: ToolProgress, token?: CancellationToken): Promise<MCP.CallToolResult>;
    _callWithProgress(params: Record<string, unknown>, progress: ToolProgress, token?: CancellationToken, allowRetry?: boolean): Promise<MCP.CallToolResult>;
    compare(other: IMcpTool): number;
}
export {};
