import { CancellationToken } from "@codingame/monaco-vscode-api/vscode/vs/base/common/cancellation";
import { Disposable } from "@codingame/monaco-vscode-api/vscode/vs/base/common/lifecycle";
import { IInstantiationService } from "@codingame/monaco-vscode-api/vscode/vs/platform/instantiation/common/instantiation";
import { ILogger } from "@codingame/monaco-vscode-api/vscode/vs/platform/log/common/log";
import { IMcpMessageTransport } from "@codingame/monaco-vscode-api/vscode/vs/workbench/contrib/mcp/common/mcpRegistryTypes";
import { MCP } from "./modelContextProtocol.js";
export interface McpRoot {
    uri: string;
    name?: string;
}
export declare class McpServerRequestHandler extends Disposable {
    private readonly launch;
    readonly logger: ILogger;
    private _nextRequestId;
    private readonly _pendingRequests;
    private _hasAnnouncedRoots;
    private _roots;
    set roots(roots: MCP.Root[]);
    private _serverInit;
    get capabilities(): MCP.ServerCapabilities;
    get serverInfo(): MCP.Implementation;
    private readonly _onDidReceiveCancelledNotification;
    readonly onDidReceiveCancelledNotification: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<MCP.CancelledNotification>;
    private readonly _onDidReceiveProgressNotification;
    readonly onDidReceiveProgressNotification: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<MCP.ProgressNotification>;
    private readonly _onDidChangeResourceList;
    readonly onDidChangeResourceList: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<void>;
    private readonly _onDidUpdateResource;
    readonly onDidUpdateResource: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<MCP.ResourceUpdatedNotification>;
    private readonly _onDidChangeToolList;
    readonly onDidChangeToolList: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<void>;
    private readonly _onDidChangePromptList;
    readonly onDidChangePromptList: import("@codingame/monaco-vscode-api/vscode/vs/base/common/event").Event<void>;
    static create(instaService: IInstantiationService, launch: IMcpMessageTransport, logger: ILogger, token?: CancellationToken): Promise<McpServerRequestHandler>;
    protected constructor(launch: IMcpMessageTransport, logger: ILogger);
    private sendRequest;
    private send;
    private sendRequestPaginated;
    private sendNotification;
    private handleMessage;
    private handleResult;
    private handleError;
    private handleServerRequest;
    private handleServerNotification;
    private handleCancelledNotification;
    private handleLoggingNotification;
    private respondToRequest;
    private handlePing;
    private handleRootsList;
    private cancelAllRequests;
    dispose(): void;
    initialize(params: MCP.InitializeRequest["params"], token?: CancellationToken): Promise<MCP.InitializeResult>;
    listResources(params?: MCP.ListResourcesRequest["params"], token?: CancellationToken): Promise<MCP.Resource[]>;
    readResource(params: MCP.ReadResourceRequest["params"], token?: CancellationToken): Promise<MCP.ReadResourceResult>;
    listResourceTemplates(params?: MCP.ListResourceTemplatesRequest["params"], token?: CancellationToken): Promise<MCP.ResourceTemplate[]>;
    subscribe(params: MCP.SubscribeRequest["params"], token?: CancellationToken): Promise<MCP.EmptyResult>;
    unsubscribe(params: MCP.UnsubscribeRequest["params"], token?: CancellationToken): Promise<MCP.EmptyResult>;
    listPrompts(params?: MCP.ListPromptsRequest["params"], token?: CancellationToken): Promise<MCP.Prompt[]>;
    getPrompt(params: MCP.GetPromptRequest["params"], token?: CancellationToken): Promise<MCP.GetPromptResult>;
    listTools(params?: MCP.ListToolsRequest["params"], token?: CancellationToken): Promise<MCP.Tool[]>;
    callTool(params: MCP.CallToolRequest["params"] & MCP.Request["params"], token?: CancellationToken): Promise<MCP.CallToolResult>;
    setLevel(params: MCP.SetLevelRequest["params"], token?: CancellationToken): Promise<MCP.EmptyResult>;
    complete(params: MCP.CompleteRequest["params"], token?: CancellationToken): Promise<MCP.CompleteResult>;
}
